#include <GL/glut.h>
#include <math.h>
#include <stdlib.h>

#include "water.h"
#include "texturehandling.h"
#include "scene.h"
#include "timing.h"

/* Extern globals */
extern state appState;
extern int curr_seatex;
extern int texture[];

#define M_PI 3.14159265358979323846

/* Water surface */
void set_water(void)
{
#define ORDER 4
#define NUMKNOTS (appState.numCtrlPts + ORDER)
	GLUnurbs *nobj;
	static GLfloat *ctpnts;
	static GLfloat *knots;
	GLfloat mat_sea[] = {0.2, 0.4, 0.8, 0.1};
	GLfloat mat_lines[] = {0.0, 1.0, 1.0, 0.1};
	GLfloat mat_points[] = {1.0, 0.0, 0.0, 0.1};

	GLint i, j, currNumCtrlPts = 0;
	static GLint initialized = 0;

	/* Initializing routines
	 * Should only be performed first time this function is run
	 * or when size of control point array has changed */
	if(!initialized || currNumCtrlPts != appState.numCtrlPts) {
		if(initialized && currNumCtrlPts != appState.numCtrlPts) {
			free(ctpnts);
			free(knots);
		}
		ctpnts = malloc(sizeof(GLfloat)*appState.numCtrlPts*appState.numCtrlPts*3);
		knots = malloc(sizeof(GLfloat)*NUMKNOTS);
		
		/* Fill ctpnts */
		for(i=0; i < appState.numCtrlPts; i++) {
			for(j=0; j < appState.numCtrlPts; j++) {
				ctpnts[(i*appState.numCtrlPts*3) + (j*3)] = -((appState.numCtrlPts-1)/2) + j;
				ctpnts[(i*appState.numCtrlPts*3) + (j*3) + 1] = 0;
				ctpnts[(i*appState.numCtrlPts*3) + (j*3) + 2]  = -((appState.numCtrlPts-1)/2) + i;
			}
		}
		/* Fill knots */
		for(i=0; i < NUMKNOTS; i++) {
			knots[i] = i;
		}

		/* Set as initialized */
		initialized = 1;
		currNumCtrlPts = appState.numCtrlPts;
	}

	/* Modulate ctpnts */
	modwater((GLfloat *)ctpnts, appState.numCtrlPts, appState.waveAmplitude);

	/* Display */
	glPushMatrix();
	{
		/* Scale to fit "ocean" */
		glScalef(20, 2, 20);
		glEnable(GL_BLEND);
		glEnable(GL_AUTO_NORMAL);

		/* Draw lines between controlpoints */
		if(appState.showCtrlLines) {
			glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_lines);
			for(i=0; i < appState.numCtrlPts; i++) {
				glBegin(GL_LINE_STRIP);
				for(j=0; j < appState.numCtrlPts; j++) {
					glVertex3f(ctpnts[(i*appState.numCtrlPts*3) + (j*3)],
							ctpnts[(i*appState.numCtrlPts*3) + (j*3) + 1],
							ctpnts[(i*appState.numCtrlPts*3) + (j*3) + 2]);
				}
				glEnd();
			}
		}

		/* Draw controlpoints */
		if(appState.showCtrlPts) {
			glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_points);
			glPointSize(5);
			glBegin(GL_POINTS);
			for(i=0; i < appState.numCtrlPts; i++) {
				for(j=0; j < appState.numCtrlPts; j++) {
					glVertex3f(ctpnts[(i*appState.numCtrlPts*3) + (j*3)],
							ctpnts[(i*appState.numCtrlPts*3) + (j*3) + 1],
							ctpnts[(i*appState.numCtrlPts*3) + (j*3) + 2]);
				}
			}
			glEnd();
		}

		nobj = gluNewNurbsRenderer();

		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_sea);

		if(appState.useWire)
			gluNurbsProperty(nobj, GLU_DISPLAY_MODE, GLU_OUTLINE_POLYGON);
		else
			gluNurbsProperty(nobj, GLU_DISPLAY_MODE, GLU_FILL);

		gluNurbsProperty(nobj, GLU_SAMPLING_TOLERANCE, 400);
		
		gluBeginSurface(nobj);
		gluNurbsSurface(nobj, NUMKNOTS, knots, NUMKNOTS, knots, appState.numCtrlPts*3, 3, ctpnts, ORDER, ORDER, GL_MAP2_VERTEX_3);
		gluEndSurface(nobj);

		gluDeleteNurbsRenderer(nobj);

		glDisable(GL_BLEND);
	}
	glPopMatrix();
}

/* Modify control point array */
void modwater(GLfloat *ypts, int arr_size, float ampl)
{
	static GLdouble incr = 0;
	static GLdouble prev_time = 0, curr_time = 0;
	int i, j;

	curr_time = getsec();
	incr += appState.waveSpeed * (curr_time - prev_time);
	prev_time = curr_time;

	for(i=0; i < arr_size; i++) {
		for(j=0; j < arr_size; j++) {
			switch(appState.modFunc) {
				default:
					ypts[(i*arr_size*3)+(j*3)+1] = ampl*sin(0.5*i+j+incr) +
						ampl*cos(0.5*i+j+1.01*incr) +
						ampl*sin(-1.2*i+0.8*j+0.65*incr) +
						ampl*sin(-1.2*i+0.85*j+0.64*incr);
					break;
				case 2:
					ypts[(i*arr_size*3)+(j*3)+1] = ampl*sin(i+j+incr);
					break;
				case 3:
					ypts[(i*arr_size*3)+(j*3)+1] = ampl*sin(i+j+incr) + sin(i*j*incr/20);
					break;
				case 4:
					ypts[(i*arr_size*3)+(j*3)+1] = ampl*sin(0.5*i+j+incr) + ampl*sin(-1.2*i+0.8*j+0.65*incr);
					break;
				case 5:
					ypts[(i*arr_size*3)+(j*3)+1] = ampl*sin(2.2*j+incr) + ampl*cos(i+incr) + sin(i*j);
					break;
			} 
		}
	}
}

/* The island */
void set_terrain(void)
{
	GLfloat ctrlpoints[8][8][3] = {
	{ 	  { -30.00, -21.0, 30.00} ,
	  { -7.14, -21.0, 30.00} ,
	  { -4.29, -21.0, 30.00} ,
	  { -1.43, -21.0, 30.00} ,
	  { 1.43, -21.00, 30.00} ,
	  { 4.29, -21.0, 30.00} ,
	  { 7.14, -21.0, 30.00} ,
	  { 30.00, -21.0, 30.00} 	}, 
	{ 	  { -30.00, -21.0, 7.14} ,
	  { -7.14, 21.0, 7.14} ,
	  { -4.29, 25.0, 7.14} ,
	  { -1.43, 4.0, 7.14} ,
	  { 1.43, 20.0, 7.14} ,
	  { 4.29, 1.0, 7.14} ,
	  { 7.14, 1.0, 7.14} ,
	  { 30.00, -21.0, 7.14} 	}, 
	{ 	  { -30.00, -21.0, 4.29} ,
	  { -7.14, 21.0, 4.29} ,
	  { -4.29, 12.9, 4.29} ,
	  { -1.43, 1.9, 4.29} ,
	  { 1.43, 11.9, 4.29} ,
	  { 4.29, 1.9, 4.29} ,
	  { 7.14, 1.0, 4.29} ,
	  { 30.00, -21.0, 4.29} 	}, 
	{ 	  { -30.00, -21.0, 1.43} ,
	  { -7.14, 21.0, 1.43} ,
	  { -4.29, 1.9, 1.43} ,
	  { -1.43, 9.9, 1.43} ,
	  { 1.43, 11.9, 1.43} ,
	  { 4.29, 11.9, 1.43} ,
	  { 7.14, 1.0, 1.43} ,
	  { 30.00, -21.0, 1.43} 	}, 
	{ 	  { -30.00, -21.0, -1.43} ,
	  { -7.14, 21.0, -1.43} ,
	  { -4.29, 14.9, -1.43} ,
	  { -1.43, -24.9, -1.43} ,
	  { 1.43, -22.9, -1.43} ,
	  { 4.29, 16.9, -1.43} ,
	  { 7.14, 14.0, -1.43} ,
	  { 30.00, -21.0, -1.43} 	}, 
	{ 	  { -30.00, -21.0, -4.29} ,
	  { -7.14, 11.0, -4.29} ,
	  { -4.29, 1.9, -4.29} ,
	  { -1.43, -45.9, -4.29} ,
	  { 1.43, -5.9, -4.29} ,
	  { 4.29, 12.9, -4.29} ,
	  { 7.14, 21.0, -4.29} ,
	  { 30.00, -21.0, -4.29} 	}, 
	{ 	  { -30.00, -21.0, -7.14} ,
	  { -7.14, 33.0, -7.14} ,
	  { -4.29, 10.0, -7.14} ,
	  { -1.43, 39.0, -7.14} ,
	  { 1.43, 27.0, -7.14} ,
	  { 4.29, 20.0, -7.14} ,
	  { 7.14, 25.0, -7.14} ,
	  { 30.00, -21.0, -7.14} 	}, 
	{ 	  { -30.00, -21.0, -30.00} ,
	  { -7.14, -21.0, -30.00} ,
	  { -4.29, -21.0, -30.00} ,
	  { -1.43, -21.0, -30.00} ,
	  { 1.43, -21.0, -30.00} ,
	  { 4.29, -21.0, -30.00} ,
	  { 7.14, -21.0, -30.00} ,
	  { 30.00, -21.0, -30.00} 	}, 
	};

	GLdouble texCtlPts[2][2][2] = {
		{{ 0, 0 }, { 0, 20 }},
		{{ 20, 0 }, { 20, 20 }}
	};

	GLfloat mat_horizon[] = {1, 1, 1, 1};
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_horizon);

	/* Texture */
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_MAP2_TEXTURE_COORD_2);
	glBindTexture(GL_TEXTURE_2D, texture[TEX_GRASS4]);
	glMap2d(GL_MAP2_TEXTURE_COORD_2, 0.0, 1.0, 4, 2,
			0.0, 1.0, 2, 2, &texCtlPts[0][0][0]);
	
	/* Draw bezier-island */
	glMap2f(GL_MAP2_VERTEX_3, 0, 1, 3, 8,
			0, 1, 24, 8, &ctrlpoints[0][0][0]);
	
	glEnable(GL_MAP2_VERTEX_3);
	glEnable(GL_AUTO_NORMAL);
	
	glMapGrid2f(40, 0.0, 1.0, 40, 0.0, 1.0);
	glEvalMesh2(GL_FILL, 0, 40, 0, 40);


	/* Plattform of island */
	glBegin(GL_QUADS);
	{
		glTexCoord2f(0, 50); glVertex3f(-EDGE, -18, -EDGE);
		glTexCoord2f(50, 50); glVertex3f(EDGE, -18, -EDGE);
		glTexCoord2f(50, 0); glVertex3f(EDGE, -18, EDGE);
		glTexCoord2f(0, 0); glVertex3f(-EDGE, -18, EDGE);
	}
	glEnd();

	glDisable(GL_TEXTURE_2D);
}
