#include <stdlib.h>
#include <stdio.h>
#include <GL/glut.h>

#ifndef _WIN32
#include <unistd.h>
#endif

#include "water.h"
#include "timing.h"
#include "texturehandling.h"
#include "scene.h"

/* Globals */
state appState;
int curr_seatex = TEX_SEA1;
	
/* File-scope globals */
static GLint framecount = 0, frame_time = 0;
static GLdouble curr_time, prev_time;
static GLfloat angle = 0, distance = 160, height = 10, offset = 0; 
static GLfloat white_light[] = {1.0, 1.0, 1.0, 1.0};
static GLfloat light_position[] = {0.0, 2, 20.0, 0.0};

/* Externals */
extern int texture[];

void display(void)
{
	prev_time = curr_time;

	/* Reset matrix */
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	glMatrixMode(GL_MODELVIEW);
	glClearColor(1, 1, 1, 1);

	/* Translate */
	glTranslatef(0, 0, -2);
	gluLookAt(offset, height, distance, 0, 0, 0, 0, 1, 0);
	glRotatef(angle, 0, 1, 0);

	/* Reposition lights */
	glLightfv(GL_LIGHT0, GL_POSITION,light_position);

	/* Island */
	if(appState.showEnviron)
		set_terrain();

	/* Water surface */
	set_water();

	glutSwapBuffers();
	framecount++;
}

void init_state(void)
{
	angle = 0;
	
	appState.numCtrlPts = 12;
	appState.waveAmplitude = 0.4;
	appState.waveSpeed = 4;
	appState.modFunc = 1;
	appState.useWire = true;
	appState.circulate = false;
	appState.showCtrlLines = false;
	appState.showCtrlPts = false;
	appState.showEnviron = false;
	appState.verbose = false;
}

void idle(void)
{
	/* Framerate */
	curr_time = getsec();
	if((int)curr_time != frame_time) {
		if(appState.verbose)
			printf("%d frames per second.\n", framecount);
		framecount = 0;
		frame_time = curr_time;
	}

	/* Rotate polygon */
	if(appState.circulate) {
		if(angle > 360)
			angle = 0;
		else
			angle += (curr_time - prev_time)*10;
	}

	/* Redisplay */
	glutPostRedisplay();
}

void init(void)
{
	glDepthFunc(GL_LESS);
	glShadeModel(GL_SMOOTH);
	glEnable(GL_DEPTH_TEST);
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	glEnable(GL_NORMALIZE);

	/* Lights */
	glLightfv(GL_LIGHT0, GL_DIFFUSE, white_light);
	glLightfv(GL_LIGHT0, GL_SPECULAR, white_light);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	/* Timing */
	curr_time = getsec();
	frame_time = curr_time;

	/* Textures */
	LoadGLTextures();
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

	/* Set initial state */
	init_state();
}	

void keyhandler(unsigned char key, int x, int y)
{
	switch(key) {
		/* Quit */
		case 27:
			exit(0); break;
		/* Switch textures on plain sea */
		case '1':
			appState.modFunc = 1; break;
		case '2':
			appState.modFunc = 2; break;
		case '3':
			appState.modFunc = 3; break;
		case '4':
			appState.modFunc = 4; break;
		case '5':
			appState.modFunc = 5; break;
		/* Movement */
		case 'w':
			distance -= 1; break;
		case 's':
			distance += 1; break;
		case 'd':
			offset += 0.5; break;
		case 'a':
			offset -= 0.5; break;
		case 'q':
			height += 1; break;
		case 'e':
			height -= 1; break;
		/* Change appState */
		case ' ':
			appState.circulate = !appState.circulate; break;
		case 'k':
			appState.useWire= !appState.useWire; break;
		case 'c':
			appState.showCtrlLines = !appState.showCtrlLines; break;
		case 'p':
			appState.showCtrlPts = !appState.showCtrlPts; break;
		case 'l':
			appState.showEnviron = !appState.showEnviron; break;
		case 'v':
			appState.verbose = !appState.verbose; break;
		case '+':
			appState.waveAmplitude += 0.02;
			if(appState.verbose)
				printf("Wave amplitude: %f\n", appState.waveAmplitude);
			break;
		case '-':
			appState.waveAmplitude -= 0.02;
			if(appState.verbose)
				printf("Wave amplitude: %f\n", appState.waveAmplitude);
 			break;
		case '*':
			appState.waveSpeed += 0.06;
			if(appState.verbose)
				printf("Wave speed: %f\n", appState.waveSpeed);
			break;
		case '/':
			appState.waveSpeed -= 0.06;
			if(appState.verbose)
				printf("Wave speed: %f\n", appState.waveSpeed);
			break;
		case 'm':
			appState.numCtrlPts += 1;
			if(appState.verbose)
				printf("Number of control points: %d\n", appState.numCtrlPts);
			break;
		case 'n':
			if(appState.numCtrlPts > 2)
				appState.numCtrlPts -= 1;
			if(appState.verbose)
				printf("Number of control points: %d\n", appState.numCtrlPts);
			break;
		case 'r':
			init_state(); break;
	}
}

void reshape(int w, int h)
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0f, (GLdouble)w/h, 0.1, 300.0);
	glMatrixMode(GL_MODELVIEW);
	glViewport(0, 0, w, h);
}

int main(int argc, char *argv[])
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH | GLUT_MULTISAMPLE | GLUT_STENCIL);
	glutInitWindowPosition(20, 20);
	glutInitWindowSize(900, 600);
	glutCreateWindow("Water");
	init();

	glutKeyboardFunc(keyhandler);
	glutReshapeFunc(reshape);
	glutDisplayFunc(display);
	glutIdleFunc(idle);

	glutMainLoop();

	return(0);
}
